note
	description: "Summary description for {TEST_ACCOUNT}."
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	TEST_ACCOUNT
inherit
	ES_TEST
create
	make

feature -- Constructor for adding tests
	make
		do
			-- Tests where NO contract violations are expected
			add_boolean_case (agent test_withdraw)

			-- Tests where contract violations are expected
			add_violation_case_with_tag ("non_negative_amount", agent test_withdraw_precondition_violation)
			add_violation_case_with_tag ("balance_deducted", agent test_withdraw_postcondition_violation)
		end

feature -- Test queries (test to succeed)

	test_withdraw: BOOLEAN
		-- This is a test query becuase it has a return value.
		local
			acc: ACCOUNT
		do
			comment("test: normal execution of withdraw feature")
			create {ACCOUNT} acc.make("Alan", 100)
			-- after this, we want to know if alan has
			-- got 100 initial balance

			-- Imagine that acc.make always initializes
			-- balance as 90
			Result := acc.balance = 100
			-- this very first re-assignment
			-- will change Result to false

			-- we need such an assertion at each
			-- intermediate point to make sure
			-- Result is never false.
			check Result end

			acc.withdraw (20)
			Result := acc.balance = 80
		end

feature -- Test commands (test to fail)

	test_withdraw_precondition_violation
		local
			acc: ACCOUNT
		do
			comment ("test: expected precondition violation of withdraw")
			create {ACCOUNT} acc.make ("Mark", 100)
			-- Precondition Violation
      		-- with tag "non_negative_amount" is expected.
			acc.withdraw (-1000000)
		end

	test_withdraw_postcondition_violation
		local
			acc: BAD_ACCOUNT_WITHDRAW
		do
			comment ("test: expected postcondition violation of withdraw")
			create acc.make ("Alan", 100)
			-- Postcondition Violation
			-- with tag "balance_deduced" to occur.
			acc.withdraw (50)
		end

end
